package com.cultural.heritage.controller.address;

import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.contacts.ContactsAddRequest;
import com.cultural.heritage.model.dto.contacts.ContactsUpdateRequest;
import com.cultural.heritage.model.entity.Contacts;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.service.address.ContactsService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/contacts")
@Slf4j
@Tag(name = "联系人信息模块")
public class ContactsController {


    @Resource
    private ContactsService contactsService;


    @Resource
    private UserService userService;


    /**
     * 添加联系人
     * @param contactsAddRequest 联系人添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户添加联系人信息", description = "参数：联系人添加请求体，权限：所有人，方法名：addContacts")
    public BaseResponse<Boolean> addContacts(@RequestBody ContactsAddRequest contactsAddRequest, HttpServletRequest request) {
        if (contactsAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Contacts contacts = new Contacts();
        BeanUtils.copyProperties(contactsAddRequest, contacts);
        contacts.setUserId(userId);
        contactsService.validContacts(contacts, false);
        contactsService.verifyIsDefault(contacts);
        boolean result = contactsService.save(contacts);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 更新联系人
     * @param contactsUpdateRequest  联系人更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "小程序端用户更新联系人信息", description = "参数:联系人更新请求体，权限：所有人，方法名：updateContacts")
    public BaseResponse<Boolean> updateContacts(@RequestBody ContactsUpdateRequest contactsUpdateRequest, HttpServletRequest request) {
        if (contactsUpdateRequest == null || contactsUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Contacts contacts = new Contacts();
        BeanUtils.copyProperties(contactsUpdateRequest, contacts);
        contacts.setUserId(userId);
        contactsService.validContacts(contacts, true);
        contactsService.verifyIsDefault(contacts);
        boolean result = contactsService.updateById(contacts);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }




    /**
     * 删除联系人
     * @param contactsDeleteRequest  联系人删除请求体
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "小程序端用户删除联系人信息", description = "参数：联系人删除请求体，权限：所有人，方法名：deleteContacts")
    public BaseResponse<Boolean> deleteContacts(@RequestBody CommonRequest contactsDeleteRequest, HttpServletRequest request) {
        if (contactsDeleteRequest == null || contactsDeleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = contactsDeleteRequest.getId();
        boolean result = contactsService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 查询用户联系人信息
     * @return  用户联系人列表
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询联系人信息", description = "参数：无，权限：所有人，方法名：listUserContacts")
    public BaseResponse<List<Contacts>> listUserContacts(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        List<Contacts> list = contactsService.getUserContactsById(userId);
        return ResultUtils.success(list);
    }







}
