package com.cultural.heritage.controller.file;

import cn.hutool.core.io.FileUtil;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.dto.file.UploadFileRequest;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.enums.FileUploadBizEnum;
import com.cultural.heritage.service.file.IHweiYunOBSService;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.utils.RegexUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;

@RestController
@RequestMapping("/file")
@Slf4j
@Tag(name = "文件上传接口")
public class FileController {


	@Resource
	private UserService userService;


	@Resource
	private IHweiYunOBSService iHweiYunOBSService;




	/**
	 * Web端文件上传
	 * @param multipartFile	文件对象
	 * @param uploadFileRequest	业务类型
	 * @return	PutObjectResult
	 */
	@PostMapping("/uploadFile")
	@Operation(summary = "Web端文件上传", description = "参数：文件对象（multipartFile), 业务类型(biz)，权限：管理员，方法名：uploadFile")
	public BaseResponse<String> uploadFile(@RequestPart("file") MultipartFile multipartFile, UploadFileRequest uploadFileRequest, HttpServletRequest request) {
		// 校验用户是否登录
		User loginUser = userService.getLoginUser(request);
		if (loginUser == null) {
			throw new BusinessException(ErrorCode.NOT_LOGIN_ERROR, "未登录");
		}
		// 获取文件名
		String filePath = getFilePath(multipartFile, uploadFileRequest);
		// 格式化图片特殊字符
		filePath = RegexUtils.encodeUrl(filePath);
		// 上传文件至华为云服务器
		iHweiYunOBSService.fileUpload(multipartFile, filePath);

		return ResultUtils.success(filePath, "上传成功");
	}



	/**
	 * 文件下载接口
	 * @param objectKey 具体的文件名（含存储路径）
	 */
	@GetMapping("downloadFile")
	@Operation(summary = "文件下载", description = "参数：文件对象（具体的文件名（含存储路径）)，权限：所有人，方法名：downloadFile")
	public void downloadFile(@RequestParam String objectKey, HttpServletResponse response) {
		iHweiYunOBSService.downloadFile(objectKey, response);
	}









	/**
	 * 校验文件
	 * @param multipartFile     文件
	 * @param fileUploadBizEnum 业务类型
	 */
	private void validFile(MultipartFile multipartFile, FileUploadBizEnum fileUploadBizEnum) {
		// 文件大小
		long fileSize = multipartFile.getSize();
		// 文件后缀
		String fileSuffix = FileUtil.getSuffix(multipartFile.getOriginalFilename());
		final long LIMIT = 20 * 1024 * 1024L;
		if (fileSize > LIMIT) {
			throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件大小不能超过20MB");
		}
		if (!Arrays.asList("jpeg", "jpg", "svg", "png", "webp", "JPG").contains(fileSuffix)) {
			throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件类型错误");
		}
	}


	/**
	 * @param uploadFileRequest	业务名称
	 * @param multipartFile	文件
	 * @return	文件后缀路径
	 */
	private String getFilePath(MultipartFile multipartFile, UploadFileRequest uploadFileRequest) {
		// 获取业务名称
		String biz = uploadFileRequest.getBiz();
		FileUploadBizEnum fileUploadBizEnum = FileUploadBizEnum.getEnumByValue(biz);
		if (fileUploadBizEnum == null) {
			throw new BusinessException(ErrorCode.PARAMS_ERROR, "业务名称错误");
		}
		// 校验文件
		validFile(multipartFile, fileUploadBizEnum);
		// 文件目录：根据业务、用户来划分
		String uuid = RandomStringUtils.randomAlphabetic(8);
		String filename = uuid + "-" + multipartFile.getOriginalFilename();
		return String.format("%s/%s", fileUploadBizEnum.getValue(), filename);
	}
}
